#version 330 core

in vec2 TexCoords;
in vec3 Normal;  
in vec3 FragPos;  
in vec4 FragPosLightSpace;
  
uniform vec3 lightPos; 
uniform vec3 lightColor;
in vec3 fromLightVector;

in vec3 toCameraVector;

out vec4 color;

uniform sampler2D texture_diffuse1;
uniform sampler2D shadowMap;
uniform sampler2D dudvMap;

const float shineDamper = 20.0;
const float reflectivity = 0.6;

float ShadowCalculation(vec4 fragPosLightSpace, vec3 normalDepth,vec3 lightDirectin)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
	//vec3 tmpProj = vec3((1-projCoords.x),(1-projCoords.y),projCoords.z);

    float closestDepth = texture(shadowMap, projCoords.xy).r; 

    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;

    // Check whether current frag pos is in shadow
	//shadow acne
	//float bias = 0.005;
	float bias = max(0.05 * (1.0 - dot(normalDepth, lightDirectin)), 0.005);  
	

	float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;

	if(projCoords.z > 1.0) {
        shadow = 0.0;
	}
	
    return shadow;
}

void main()
{    

	vec3 viewVector = normalize(toCameraVector);
	// Ambient
    float ambientStrength = 0.2;
    vec3 ambientVec = ambientStrength * lightColor;
	vec3 ambient = ambientVec * vec3(texture(texture_diffuse1, TexCoords));

	// Diffuse 
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(lightPos - FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuseLight = diff * lightColor;
	vec3 diffuse = diffuseLight * vec3(texture(texture_diffuse1, TexCoords));

	//specular points, normalmap = dudvMap
	vec4 normalMapColour = texture(dudvMap,TexCoords);
	vec3 normal = vec3(normalMapColour.r,normalMapColour.b,normalMapColour.g);
	normal = normalize(normal*2.0-1.0);

	vec3 reflectedLight = reflect(normalize(fromLightVector),normal);
	float specular = max(dot(reflectedLight,viewVector),0.0);
	vec3 specularVec = vec3(0.3)*specular;

	//Shadow
	float shadow = ShadowCalculation(FragPosLightSpace,norm,lightDir); 
	vec3 colortmp = texture(texture_diffuse1, TexCoords).rgb;      
    vec3 lighting = (ambient + (1.0 - shadow) * diffuse * colortmp);  

	color = vec4(lighting, 1.0);

	//vec3 result = (ambient + diffuse + specularVec);
	//color = mix(vec4(result,1.0f), vec4(normal,1.0f),0.2);

	//color = vec4(result, 1.0);

}