#version 330 core

in vec2 TexCoords;
in vec3 Normal;
in vec3 FragPos;
in vec4 FragPosLightSpace;

uniform vec3 lightPos;
uniform vec3 lightColor;

out vec4 FragColor;

uniform sampler2D texture_diffuse1;
uniform sampler2D shadowMap;
uniform sampler2D dudvMap;


float ShadowCalculation(vec4 fragPosLightSpace, vec3 normalDepth,vec3 lightDirectin)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // Transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // Get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 

    // Get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // Check whether current frag pos is in shadow
    
	

	//shadow acne
	//float bias = 0.005;
	float bias = max(0.05 * (1.0 - dot(normalDepth, lightDirectin)), 0.005);  
	//float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;		

	float shadow = 0.0;
    vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;

	if(projCoords.z > 1.0) {
        shadow = 0.0;
	}

    return shadow; 
}

void main()
{           
    // Ambient
    float ambientStrength = 0.1f;
    vec3 ambientVec = ambientStrength * lightColor;
	vec3 ambient = ambientVec * vec3(texture(texture_diffuse1, TexCoords));

  // Diffuse 
    vec3 norm = normalize(Normal);
    vec3 lightDir = normalize(lightPos - FragPos);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuseLight = diff * lightColor;
	vec3 diffuse = diffuseLight * vec3(texture(texture_diffuse1, TexCoords));

	//Shadow
	float shadow = ShadowCalculation(FragPosLightSpace,norm,lightDir); 
	vec3 color = texture(texture_diffuse1, TexCoords).rgb;      
    vec3 lighting = (ambient + (1.0 - shadow) * diffuse * color);    
    
    FragColor = vec4(lighting, 1.0f);
}
