#version 330 core

in vec2 TexCoords;
in vec2 TexCoordsRef;
in vec4 clipspace;

in vec3 FragPos;  
in vec3 toCameraVector;
in vec3 fromLightVector;
  
out vec4 color;

uniform sampler2D texture_diffuse1;
uniform sampler2D reflectionTexture;
uniform sampler2D refractionTexture;
uniform sampler2D dudvMap;

uniform float moveFactor;

const float waveStrength = 0.2;
const float shineDamper = 20.0;
const float reflectivity = 0.6;

void main()
{ 
	vec2 ndc = (clipspace.xy/clipspace.w)/2.0+0.5;

	vec2 refractTexCoords = vec2(ndc.x,ndc.y);
	vec2 reflectTexCoords = vec2(-ndc.x,-ndc.y);

	//vec2 distortion1 = (texture(dudvMap,vec2(TexCoords.x+moveFactor,TexCoords.y)).rg * 2.0-1.0)*waveStrength;
	//vec2 distortion2 = (texture(dudvMap,vec2(-TexCoords.x+moveFactor,TexCoords.y+moveFactor)).rg * 2.0-1.0)*waveStrength;
	
	//vec2 totalDistortion= distortion1+distortion2;

	//movements in water
	vec2 distortionTexCoords =  texture(dudvMap,vec2(TexCoords.x+moveFactor,TexCoords.y)).rg * 0.1;
	distortionTexCoords = TexCoords + vec2(distortionTexCoords.x,distortionTexCoords.y+moveFactor);
	vec2 totalDistortion = (texture(dudvMap,distortionTexCoords).rg * 2.0- 1.0)*waveStrength;

	refractTexCoords += totalDistortion;
	refractTexCoords = clamp(refractTexCoords,0.001,0.999);

	reflectTexCoords += totalDistortion;
	reflectTexCoords.x = clamp(reflectTexCoords.x, -0.999, -0.001);
	reflectTexCoords.y = clamp(reflectTexCoords.y, -0.999, -0.001);

	vec3 viewVector = normalize(toCameraVector);
	float refractiveFactor = dot(viewVector, vec3(0.0,1.0,0.0));
	refractiveFactor = pow(refractiveFactor,4.0);

	vec4 reflectColor = texture(reflectionTexture,reflectTexCoords);
	vec4 refractColor = texture(refractionTexture,refractTexCoords);
	vec4 standardtex = texture(texture_diffuse1, TexCoords);

	vec4 texMixture = mix(reflectColor, refractColor, refractiveFactor);

	//specular points, normalmap = dudvMap
	vec4 normalMapColour = texture(dudvMap,distortionTexCoords);
	vec3 normal = vec3((normalMapColour.r*2.0)-1.0,normalMapColour.b,(normalMapColour.g*2.0)-1.0);
	normal = normalize(normal);
	
	vec3 reflectedLight = reflect(normalize(fromLightVector),normal);
	float specular = max(dot(reflectedLight,viewVector),0.0);
	specular = pow(specular,shineDamper);

	vec3 specularHighlights = vec3(1.0,1.0,1.0) *specular * reflectivity;
	
	vec4 higlight = vec4(specularHighlights,0.0);
	vec4 tmp_color = mix(texMixture, standardtex, 0.2);
	color = mix(tmp_color, higlight,0.1);
}